@@ -31,11 +31,15 @@
 #' If style is set to "basic", only 'lat', 'lon' and 'zoom' are shown.
 #'
 #' @examples
-#' \dontrun{
-#' leaflet() %>% addTiles() # without mouse position info
-#' leaflet() %>% addTiles() %>% addMouseCoordinates(style = "basic") # with basic mouse position info
-#' mapview(easter.egg = TRUE) # detailed mouse position info by default ;-)
-#' }
+#' library(leaflet)
+#'
+#' leaflet() %>% addProviderTiles("OpenStreetMap") # without mouse position info
+#' leaflet() %>%
+#'   addProviderTiles("OpenStreetMap") %>%
+#'   addMouseCoordinates(style = "basic") # with basic mouse position info
+#' leaflet() %>%
+#'   addProviderTiles("OpenStreetMap") %>%
+#'   addMouseCoordinates() # with detailed mouse position info
 #'
 #'
 #' @export addMouseCoordinates
@@ -85,49 +89,50 @@ addMouseCoordinates <- function(map, style = c("detailed", "basic"),
       "
       function(el, x, data) {
 
-      // get the leaflet map
-      var map = this; //HTMLWidgets.find('#' + el.id);
+        // get the leaflet map
+        var map = this; //HTMLWidgets.find('#' + el.id);
 
-      // we need a new div element because we have to handle
-      // the mouseover output separately
-      // debugger;
-      function addElement () {
-      // generate new div Element
-      var newDiv = $(document.createElement('div'));
-      // append at end of leaflet htmlwidget container
-      $(el).append(newDiv);
-      //provide ID and style
-      newDiv.addClass('lnlt');
-      newDiv.css({
-      'position': 'relative',
-      'bottomleft':  '0px',
-      'background-color': 'rgba(255, 255, 255, 0.7)',
-      'box-shadow': '0 0 2px #bbb',
-      'background-clip': 'padding-box',
-      'margin': '0',
-      'padding-left': '5px',
-      'color': '#333',
-      'font': '9px/1.5 \"Helvetica Neue\", Arial, Helvetica, sans-serif',
-      });
-      return newDiv;
-      }
+        // we need a new div element because we have to handle
+        // the mouseover output separately
+        // debugger;
+        function addElement () {
+          // generate new div Element
+         var newDiv = $(document.createElement('div'));
+          // append at end of leaflet htmlwidget container
+          $(el).append(newDiv);
+          //provide ID and style
+          newDiv.addClass('lnlt');
+          newDiv.css({
+            'position': 'relative',
+            'bottomleft':  '0px',
+            'background-color': 'rgba(255, 255, 255, 0.7)',
+            'box-shadow': '0 0 2px #bbb',
+            'background-clip': 'padding-box',
+            'margin': '0',
+            'padding-left': '5px',
+            'color': '#333',
+            'font': '9px/1.5 \"Helvetica Neue\", Arial, Helvetica, sans-serif',
+            'z-index': '700',
+          });
+          return newDiv;
+        }
 
-      // check for already existing lnlt class to not duplicate
-      var lnlt = $(el).find('.lnlt');
+        // check for already existing lnlt class to not duplicate
+        var lnlt = $(el).find('.lnlt');
 
-      if(!lnlt.length) {
-      lnlt = addElement();
-      //$(el).keypress(function (e) {
-      //  if (e.which == 32 || event.keyCode == 32) {
-      //    alert('space key is pressed');
-      //  }
-      //});
-      // grab the special div we generated in the beginning
-      // and put the mousmove output there
-      map.on('mousemove', function (e) {
-      lnlt.text(", txt, ");
-      });
-      };
+        if(!lnlt.length) {
+          lnlt = addElement();
+          //$(el).keypress(function (e) {
+          //  if (e.which == 32 || event.keyCode == 32) {
+          //    alert('space key is pressed');
+          //  }
+          //});
+          // grab the special div we generated in the beginning
+          // and put the mousmove output there
+          map.on('mousemove', function (e) {
+            lnlt.text(", txt, ");
+          });
+        };
       }
       "
   )
@@ -135,6 +140,19 @@ addMouseCoordinates <- function(map, style = c("detailed", "basic"),
   map
 }
 
+
+removeMouseCoordinates = function(map) {
+  if (inherits(map, "mapview")) map = mapview2leaflet(map)
+
+  rc = map$jsHooks$render
+  rc_lnlt = lapply(rc, grep, pattern = "lnlt")
+  for (i in seq_along(map$jsHooks$render)) {
+    map$jsHooks$render[[i]][rc_lnlt[[i]]] = NULL
+  }
+
+  return(map)
+}
+
 ##############################################################################
 
 
@@ -155,16 +173,17 @@ addMouseCoordinates <- function(map, style = c("detailed", "basic"),
 #' @param add logical. Whether to add the button to the map (mainly for internal use).
 #'
 #' @examples
-#' \dontrun{
+#' library(leaflet)
 #' library(raster)
 #'
-#' m <- leaflet() %>% addTiles() %>% addCircleMarkers(data = breweries91) %>%
-#'   addHomeButton(extent(breweries91), "breweries91")
+#' m <- leaflet() %>%
+#'   addProviderTiles("OpenStreetMap") %>%
+#'   addCircleMarkers(data = breweries) %>%
+#'   addHomeButton(extent(breweries), "breweries")
 #' m
 #'
 #' ## remove the button
 #' removeHomeButton(m)
-#' }
 #'
 #'
 #' @export addHomeButton
@@ -176,6 +195,12 @@ addHomeButton <- function(map, ext, layer.name = "layer",
   if (inherits(map, "mapview")) map <- mapview2leaflet(map)
   stopifnot(inherits(map, "leaflet"))
 
+  # drop names in case extent of sf object
+  ext@xmin = unname(ext@xmin)
+  ext@xmax = unname(ext@xmax)
+  ext@ymin = unname(ext@ymin)
+  ext@ymax = unname(ext@ymax)
+
   hb <- try(getCallEntryFromMap(map, "addHomeButton"), silent = TRUE)
   if (!inherits(hb, "try-error") & length(hb) == 1) {
     ext_coords <- unlist(map$x$calls[[hb]][["args"]][1:4])
@@ -264,7 +289,7 @@ leafletHomeButtonDependencies <- function() {
 #' @param height height of the rendered image in pixels.
 #'
 #' @examples
-#' \dontrun{
+#' library(leaflet)
 #' ## default position is topleft next to zoom control
 #'
 #' img <- "https://www.r-project.org/logo/Rlogo.svg"
@@ -277,7 +302,6 @@ leafletHomeButtonDependencies <- function() {
 #' leaflet() %>% addTiles() %>% addLogo(img, src = "local", alpha = 0.3)
 #'
 #' ## dancing banana gif :-)
-#' library(magick)
 #' m <- mapview(breweries91)
 #'
 #' addLogo(m, "https://jeroenooms.github.io/images/banana.gif",
@@ -287,8 +311,6 @@ leafletHomeButtonDependencies <- function() {
 #'         width = 100,
 #'         height = 100)
 #'
-#' }
-#'
 #'
 #' @export addLogo
 #' @name addLogo
@@ -483,10 +505,11 @@ remoteImage <- function(img, alpha, url, width, height) {
 #' Type agnositc version of \code{leaflet::add*} functions.
 #'
 #' @description
-#' Add simple features geomertries from \code{\link[sf]{sf}}
+#' Add simple features geometries from \code{\link[sf]{sf}}
 #'
 #' @param map A \code{leaflet} or \code{mapview} map.
 #' @param data A \code{sf} object to be added to the \code{map}.
+#' @param pane The name of the map pane for the features to be rendered in.
 #' @param ... Further arguments passed to the respective \code{leaflet::add*}
 #' functions. See \code{\link{addCircleMarkers}}, \code{\link{addPolylines}}
 #' and \code{\link{addPolygons}}.
@@ -495,41 +518,42 @@ remoteImage <- function(img, alpha, url, width, height) {
 #' A leaflet \code{map} object.
 #'
 #' @examples
-#' \dontrun{
-#' leaflet() %>% addTiles() %>% addCircleMarkers(data = breweries)
-#' leaflet() %>% addTiles() %>% addFeatures(data = breweries)
+#' library(leaflet)
 #'
-#' leaflet() %>% addTiles() %>% addPolylines(data = atlStorms2005)
-#' leaflet() %>% addTiles() %>% addFeatures(atlStorms2005)
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addCircleMarkers(data = breweries)
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addFeatures(data = breweries)
 #'
-#' leaflet() %>% addTiles() %>% addPolygons(data = franconia)
-#' leaflet() %>% addTiles() %>% addFeatures(franconia)
-#' }
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addPolylines(data = atlStorms2005)
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addFeatures(atlStorms2005)
+#'
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addPolygons(data = franconia)
+#' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addFeatures(franconia)
 #'
 #' @export addFeatures
 #' @name addFeatures
 #' @rdname addFeatures
 addFeatures <- function(map,
                         data,
+                        pane = "overlayPane",
                         ...) {
 
   if (inherits(data, "Spatial")) data = sf::st_as_sf(data)
 
   switch(getSFClass(sf::st_geometry(data)),
-         sfc_POINT           = addPointFeatures(map, data, ...),
-         sfc_MULTIPOINT      = addPointFeatures(map, data, ...),
-         sfc_LINESTRING      = addLineFeatures(map, data, ...),
-         sfc_MULTILINESTRING = addLineFeatures(map, data, ...),
-         sfc_POLYGON         = addPolygonFeatures(map, data, ...),
-         sfc_MULTIPOLYGON    = addPolygonFeatures(map, data, ...),
-         sfc_GEOMETRY        = addGeometry(map, data, ...),
-         POINT               = addPointFeatures(map, data, ...),
-         MULTIPOINT          = addPointFeatures(map, data, ...),
-         LINESTRING          = addLineFeatures(map, data, ...),
-         MULTILINESTRING     = addLineFeatures(map, data, ...),
-         POLYGON             = addPolygonFeatures(map, data, ...),
-         MULTIPOLYGON        = addPolygonFeatures(map, data, ...),
-         GEOMETRY            = addGeometry(map, data, ...))
+         sfc_POINT           = addPointFeatures(map, data, pane, ...),
+         sfc_MULTIPOINT      = addPointFeatures(map, data, pane, ...),
+         sfc_LINESTRING      = addLineFeatures(map, data, pane, ...),
+         sfc_MULTILINESTRING = addLineFeatures(map, data, pane, ...),
+         sfc_POLYGON         = addPolygonFeatures(map, data, pane, ...),
+         sfc_MULTIPOLYGON    = addPolygonFeatures(map, data, pane, ...),
+         sfc_GEOMETRY        = addGeometry(map, data, pane, ...),
+         POINT               = addPointFeatures(map, data, pane, ...),
+         MULTIPOINT          = addPointFeatures(map, data, pane, ...),
+         LINESTRING          = addLineFeatures(map, data, pane, ...),
+         MULTILINESTRING     = addLineFeatures(map, data, pane, ...),
+         POLYGON             = addPolygonFeatures(map, data, pane, ...),
+         MULTIPOLYGON        = addPolygonFeatures(map, data, pane, ...),
+         GEOMETRY            = addGeometry(map, data, pane, ...))
 
 }
 
@@ -541,44 +565,51 @@ addFeatures <- function(map,
 
 mw = 800
 
-### Point Features ========================================================
+### Point Features
 addPointFeatures <- function(map,
                              data,
+                             pane,
                              ...) {
   garnishMap(map, leaflet::addCircleMarkers,
              data = sf::st_zm(sf::st_cast(data, "POINT")),
              popupOptions = popupOptions(maxWidth = mw,
                                          closeOnClick = TRUE),
+             options = leafletOptions(pane = pane),
              ...)
 }
 
-### Line Features =========================================================
+### Line Features
 addLineFeatures <- function(map,
                             data,
+                            pane,
                             ...) {
   garnishMap(map, leaflet::addPolylines,
              data = sf::st_zm(data),
              popupOptions = popupOptions(maxWidth = mw,
                                          closeOnClick = TRUE),
+             options = leafletOptions(pane = pane),
              ...)
 }
 
-### PolygonFeatures =======================================================
+### PolygonFeatures
 addPolygonFeatures <- function(map,
                                data,
+                               pane,
                                ...) {
   garnishMap(map, leaflet::addPolygons,
              data = sf::st_zm(data),
              popupOptions = popupOptions(maxWidth = mw,
                                          closeOnClick = TRUE),
+             options = leafletOptions(pane = pane),
              ...)
 }
 
-### GeometryCollections ===================================================
+### GeometryCollections
 addGeometry = function(map,
                        data,
+                       pane,
                        ...) {
-  ls = list(...)
+  ls = append(list(pane), list(...))
   if (!is.null(ls$label))
     label = split(ls$label, f = as.character(sf::st_dimension(data)))
   if (!is.null(ls$popup))
@@ -631,6 +662,7 @@ addGeometry = function(map,
 #'   to 'mousemove'.
 #' @param digits the number of digits to be shown in the display field.
 #' @param position where to place the display field. Default is 'topright'.
+#' @param prefix a character string to be shown as prefix for the layerId.
 #' @param ... currently not used.
 #'
 #' @return
@@ -660,6 +692,7 @@ addImageQuery = function(map,
                          type = c("mousemove", "click"),
                          digits,
                          position = 'topright',
+                         prefix = 'Layer',
                          ...) {
 
   if (inherits(map, "mapview")) map = mapview2leaflet(map)
@@ -700,7 +733,8 @@ addImageQuery = function(map,
   ctrlid = ctrlid[imctrl]
 
   if (length(ctrlid) == 0) {
-    map = addControl(map, NULL, layerId = 'imageValues', position = position)
+    # must add empty character instead of NULL for html with addControl
+    map = addControl(map, html = "", layerId = 'imageValues', position = position)
   }
 
   map$dependencies <- c(map$dependencies,
@@ -723,7 +757,7 @@ addImageQuery = function(map,
         'function(el, x, data) {
         var map = this;
         map.on("', type, '", function (e) {
-          rasterPicker.pick(e, x, ', digits, ');
+          rasterPicker.pick(e, x, ', digits, ', "', prefix, ' ");
         });
       }'
       )
@@ -734,3 +768,221 @@ addImageQuery = function(map,
 }
 
 ##############################################################################
+
+
+
+### addStaticLabels ##########################################################
+##############################################################################
+#' Add static labels to \code{leaflet} or \code{mapview} objects
+#'
+#' @description
+#' Being a wrapper around \code{\link[leaflet]{addLabelOnlyMarkers}}, this
+#' function provides a smart-and-easy solution to add custom text labels to an
+#' existing \code{leaflet} or \code{mapview} map object.
+#'
+#' @param map A \code{leaflet} or \code{mapview} object.
+#' @param data A \code{sf} or \code{Spatial*} object used for label placement,
+#' defaults to the locations of the first dataset in 'map'.
+#' @param label The labels to be placed at the positions indicated by 'data' as
+#' \code{character}, or any vector that can be coerced to this type.
+#' @param group the group of the static labels layer.
+#' @param layerId the layerId of the static labels layer.
+#' @param ... Additional arguments passed to
+#' \code{\link[leaflet]{addLabelOnlyMarkers}}.
+#'
+#' @return
+#' A labelled \strong{mapview} object.
+#'
+#' @author
+#' Florian Detsch
+#'
+#' @seealso
+#' \code{\link[leaflet]{addLabelOnlyMarkers}}.
+#'
+#' @examples
+#' \dontrun{
+#' ## leaflet label display options
+#' library(leaflet)
+#'
+#' lopt = labelOptions(noHide = TRUE,
+#'                     direction = 'top',
+#'                     textOnly = TRUE)
+#'
+#' ## point labels
+#' m1 = mapview(breweries)
+#' l1 = addStaticLabels(m1,
+#'                      label = breweries$number.of.types,
+#'                      labelOptions = lopt)
+#' l1
+#'
+#' ## polygon centroid labels
+#' m2 = mapview(franconia)
+#' l2 = addStaticLabels(m2,
+#'                      label = franconia$NAME_ASCI,
+#'                      labelOptions = lopt)
+#' l2
+#'
+#' ## custom labels
+#' m3 = m2 + m1
+#' l3 = addStaticLabels(m3,
+#'                      data = franconia,
+#'                      label = franconia$NAME_ASCI,
+#'                      labelOptions = lopt)
+#' l3
+#' }
+#'
+#' @export addStaticLabels
+#' @name addStaticLabels
+addStaticLabels = function(map,
+                           data,
+                           label,
+                           group = NULL,
+                           layerId = NULL,
+                           ...) {
+
+  if (inherits(map, "mapview") & missing(data)) {
+    data = map@object[[1]]
+    if (is.null(group)) {
+      group = getLayerNamesFromMap(map@map)[1]
+    } else {
+      group = NULL
+    }
+  }
+
+  dots = list(...)
+  min_opts = list(noHide = TRUE,
+                  direction = "top",
+                  textOnly = TRUE)
+
+  dots = append(dots, min_opts)
+
+  if (inherits(map, "mapview")) map = mapview2leaflet(map)
+
+  ## 'Raster*' locations not supported so far -> error
+  if (inherits(data, "Raster")) {
+    stop(paste("'Raster*' input is not supported, yet."
+               , "Please refer to ?addLabels for compatible input formats.\n"),
+         call. = FALSE)
+  }
+
+  ## if input is 'Spatial*', convert to 'sf'
+  if (inherits(data, "Spatial")) {
+    data = sf::st_as_sf(data)
+  }
+
+  if (missing(label)) label = makeLabels(data, NULL)
+  #   {
+  #   sf_col = attr(data, "sf_column")
+  #   if (inherits(data, "sf")) {
+  #     if (ncol(data) == 2) {
+  #       colnm = setdiff(colnames(data), sf_col)
+  #       label = data[[colnm]]
+  #     } else {
+  #       label = seq(nrow(data))
+  #     }
+  #   } else {
+  #     label = seq(length(data))
+  #   }
+  # }
+
+  if (getGeometryType(data) == "ln") {
+    crds = as.data.frame(sf::st_coordinates(data))
+    crds_lst = split(crds, crds[[ncol(crds)]])
+    mat = do.call(rbind, lapply(seq(crds_lst), function(i) {
+      crds_lst[[i]][sapply(crds_lst, nrow)[i], c("X", "Y")]
+    }))
+  } else {
+    mat = sf::st_coordinates(suppressWarnings(sf::st_centroid(data)))
+  }
+
+  ## add labels to map
+  map = garnishMap(leaflet::addLabelOnlyMarkers,
+                   map = map,
+                   lng = mat[, 1],
+                   lat = mat[, 2],
+                   label = as.character(label),
+                   group = group,
+                   layerId = layerId,
+                   labelOptions = dots)
+  # map = leaflet::addLabelOnlyMarkers(map,
+  #                                    lng = mat[, 1],
+  #                                    lat = mat[, 2],
+  #                                    label = as.character(label),
+  #                                    ...)
+
+  return(map)
+}
+
+
+##############################################################################
+
+### addStaticLabels ##########################################################
+##############################################################################
+#' Add additional panes to leaflet map to control layer order
+#'
+#' @description
+#' map panes can be created by supplying a name and a zIndex to control layer
+#' ordering. We recommend a \code{zIndex} value between 400 (the default
+#' overlay pane) and 500 (the default shadow pane). You can then use this pane
+#' to render overlays (points, lines, polygons) by setting the \code{pane}
+#' argument in \code{leafletOptions}. This will give you control
+#' over the order of the layers, e.g. points always on top of polygons.
+#' If two layers are provided to the same pane, overlay will be determined by
+#' order of adding. See examples below.
+#' See \url{http://www.leafletjs.com/reference-1.3.0.html#map-pane} for details.
+#'
+#' @param map A \code{leaflet} or \code{mapview} object.
+#' @param name The name of the new pane (refer to this in \code{leafletOptions}.
+#' @param zIndex The zIndex of the pane. Panes with higher index are rendered
+#' above panes with lower indices.
+#'
+#' @examples
+#' library(leaflet)
+#' library(mapview)
+#'
+#' ## points above polygons
+#' leaflet() %>%
+#'   addTiles() %>%
+#'   addMapPane("polygons", zIndex = 410) %>%
+#'   addMapPane("points", zIndex = 420) %>%
+#'   addPolygons(data = franconia,
+#'               group = "pol1",
+#'               fillOpacity = 0.7,
+#'               fillColor = "green",
+#'               color = "black",
+#'               options = leafletOptions(pane = "polygons")) %>%
+#'   addPolygons(data = franconia,
+#'               group = "pol2",
+#'               color = "black",
+#'               fillColor = "purple",
+#'               fillOpacity = 0.7,
+#'               options = leafletOptions(pane = "polygons")) %>%
+#'   addCircleMarkers(data = breweries,
+#'                    group = "pts",
+#'                    color = "darkblue",
+#'                    options = leafletOptions(pane = "points")) %>%
+#'   addLayersControl(overlayGroups = c("pol1", "pol2", "pts"))
+#'
+#'
+#' @export addMapPane
+#' @name addMapPane
+#'
+addMapPane = function(map, name, zIndex) {
+
+  if (inherits(map, "mapview")) map = mapview2leaflet(map)
+
+  map$dependencies <- c(map$dependencies, leafletMapPaneDependencies())
+  leaflet::invokeMethod(map, leaflet::getMapData(map), 'createMapPane',
+                        name, zIndex)
+
+}
+
+leafletMapPaneDependencies <- function() {
+  list(
+    htmltools::htmlDependency(
+      "mapPane",
+      '0.0.1',
+      system.file("htmlwidgets/lib/pane", package = "mapview"),
+      script = c('map-pane.js')
+    ))
+}
