@@ -21,13 +21,13 @@
 #' Tim Appelhans
 #'
 #' @examples
-#' \dontrun{
+#' library(leaflet)
+#'
 #' viewExtent(poppendorf)
 #' viewExtent(breweries)
 #' viewExtent(franconia) + breweries
 #' viewExtent(trails) + trails + breweries
 #' leaflet() %>% addProviderTiles("OpenStreetMap") %>% addExtent(breweries)
-#' }
 #'
 #' @export viewExtent
 #' @name viewExtent
@@ -91,7 +91,8 @@ combineExtent = function(lst, sf = FALSE, crs = 4326) {
   # lst = list(breweries, st_as_sf(atlStorms2005), st_as_sf(gadmCHE))
   # bb = do.call(rbind, lapply(lst, sf::st_bbox))
   bb = do.call(rbind, lapply(seq(lst), function(i) {
-    if (!is.na(st_crs(lst[[i]]))) {
+
+    if (!is.na(getProjection(lst[[i]]))) {
       sf::st_bbox(sf::st_transform(sf::st_as_sfc(sf::st_bbox(lst[[i]])),
                                    crs = crs))
     } else {
