@@ -10,20 +10,15 @@ if ( !isGeneric('+') ) {
 #' the objects should be added to e1.
 #'
 #' @examples
-#' \dontrun{
-#' ### raster data ###
-#' library(sp)
-#' library(raster)
-#'
-#' m1 <- mapView(poppendorf[[5]])
-#'
-#' ### point vector data ###
-#' m2 <- mapView(breweries91)
+#' m1 <- mapView(franconia, col.regions = "red")
+#' m2 <- mapView(breweries)
 #'
 #' ### add two mapview objects
 #' m1 + m2
 #' '+'(m2, m1)
-#' }
+#'
+#' ### add layers to a mapview object
+#' m1 + breweries + poppendorf[[4]]
 #'
 #' @name +
 #' @docType methods
@@ -66,19 +61,25 @@ setMethod("+",
           signature(e1 = "mapview",
                     e2 = "ANY"),
           function (e1, e2) {
+
             nm <- deparse(substitute(e2))
-            # e1 + mapview(e2, layer.name = nm)
-            m = mapview(e2, map = e1, layer.name = nm)
-            out_obj = append(e1@object, m@object)
+            e1 + mapview(e2, layer.name = nm)
 
-            hbcalls = getCallEntryFromMap(m@map, "addHomeButton")
-            zf = grep("Zoom full", m@map$x$calls[hbcalls])
-            ind = hbcalls[zf]
-            if (length(zf) > 0) m@map$x$calls[ind] = NULL
-
-            m = addZoomFullButton(m@map, out_obj)
-            out = methods::new('mapview', object = out_obj, map = m)
-            return(out)
+            # nm <- deparse(substitute(e2))
+            # e1@map = removeMouseCoordinates(e1@map)
+            # # e1 + mapview(e2, layer.name = nm)
+            # m = mapview(e2, map = e1, layer.name = nm)
+            # print(str(m, 5))
+            # out_obj = append(e1@object, m@object)
+            #
+            # hbcalls = getCallEntryFromMap(m@map, "addHomeButton")
+            # zf = grep("Zoom full", m@map$x$calls[hbcalls])
+            # ind = hbcalls[zf]
+            # if (length(zf) > 0) m@map$x$calls[ind] = NULL
+            #
+            # m = addZoomFullButton(m@map, out_obj)
+            # out = methods::new('mapview', object = out_obj, map = m)
+            # return(out)
           }
 )
 
